#include <iostream>
#include <list>
#include <algorithm>
#include <functional>

#include "tracks.h"
#include "cddb_query.h"

struct print_track 
    : public std::unary_function<const TagLookup::Track &, void>
{
    std::ostream &os;
    unsigned int s;
    
    print_track(std::ostream &out, unsigned int spaces) 
        : os(out), s(spaces)
    { }
    
    void operator() (const TagLookup::Track &track)
    {
        for(unsigned int i = 0; i < s; ++i)
            os << ' ';
        
        unsigned int n = track.getTrackNo();
        os << (n < 10 ? " " : "") << n << ". " << track.getName() << '\n'; 
    }
}; // print_track

struct print_disc 
    : public std::unary_function<const TagLookup::Disc &, void>
{
    std::ostream &os;
    unsigned int count;
    
    print_disc(std::ostream& out) 
        : os(out), count(0)
    {
    }
    
    void operator() (const TagLookup::Disc &disc)
    {
        ++count;
        
        os << '[' << (count < 10 ? " " : "") << count << "] " << disc.getArtist() << " - " << disc.getTitle() << '\n';
        
        TagLookup::Tracks ts = disc.getTracks(); 
        
        std::for_each(ts.begin(), ts.end(), print_track(os, 5));
    }
}; // print_disc

int main()
{
    std::list<int> offsets;
    offsets.push_back(150);
    offsets.push_back(28690);
    offsets.push_back(51102);
    offsets.push_back(75910);
    offsets.push_back(102682);
    offsets.push_back(121522);
    offsets.push_back(149040);
    offsets.push_back(175772);
    offsets.push_back(204387);
    offsets.push_back(231145);
    offsets.push_back(268065);

    unsigned int len = 3822;
    
    TagLookup::CDDBQuery q(len, offsets);
    
    q.query();
    
    TagLookup::Discs d(q.getResults());
    
    for_each(d.begin(), d.end(), print_disc(std::cout));

    return 0; 
} // main()