/*
 * Copyright (c) Alex Allan <alex X kamaz.org.uk> 
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "command_line.h"

#include <list>
#include <string>
#include <algorithm>
#include <utility>

#include <iostream>

using namespace TagLookup;

template<class T>
struct Print
    : public std::unary_function<T, void>
{
    void operator() (T x)
    {
        std::cout << x << std::endl;
    }
};

int main(const int argc, const char* argv[])
{
    CommandLineArguments cla;
    
    try
    {
        CommandLineArgumentsParser clap(argc, argv);
    
        cla = clap.getArguments();
    
        for_each(cla.files.begin(), cla.files.end(), Print<std::string&>());
    }
    catch(ParseException &e)
    {
        std::cout << e.what() << std::endl;
    }

    return 0;
}

