#include <iostream>
#include <list>
#include <string>
#include <algorithm>
#include <functional>

#include <taglib/fileref.h>

#include "cddb_query.h"
#include "tag.h"

using namespace TagLookup;

struct print_track 
    : public std::unary_function<const TagLookup::Track &, void>
{
    std::ostream &os;
    unsigned int s;
    
    print_track(std::ostream &out, unsigned int spaces) 
        : os(out), s(spaces)
    { }
    
    void operator() (const TagLookup::Track &track)
    {
        for(unsigned int i = 0; i < s; ++i)
            os << ' ';
        
        unsigned int n = track.getTrackNo();
        os << (n < 10 ? " " : "") << n << ". " << track.getName() << '\n'; 
    }
}; // print_track

struct print_disc 
    : public std::unary_function<const TagLookup::Disc &, void>
{
    std::ostream &os;
    unsigned int count;
    
    print_disc(std::ostream& out) 
        : os(out), count(0)
    {
    }
    
    void operator() (const TagLookup::Disc &disc)
    {
        ++count;
        
        os << '[' << (count < 10 ? " " : "") << count << "] " << disc.getArtist() << " - " << disc.getTitle() << '\n';
        
        TagLookup::Tracks ts = disc.getTracks(); 
        
        std::for_each(ts.begin(), ts.end(), print_track(os, 5));
    }
}; // print_disc


template<class T>
struct print
    : public std::unary_function<T, void>
{
    std::string p_;
    unsigned int c_;
    
    print(const std::string & prefix)
        : p_(prefix), c_(0)
    {
    }
    
    void operator() (T x)
    {
        std::cout << p_ << c_++ << ": " << x << std::endl;
    }
};

int main()
{
    std::list<TagLib::FileRef> fl;
    
fl.push_back(TagLib::FileRef("/tmp/simple/01.mp3"));
fl.push_back(TagLib::FileRef("/tmp/simple/02.mp3"));
fl.push_back(TagLib::FileRef("/tmp/simple/03.mp3"));
fl.push_back(TagLib::FileRef("/tmp/simple/04.mp3"));
fl.push_back(TagLib::FileRef("/tmp/simple/05.mp3"));
fl.push_back(TagLib::FileRef("/tmp/simple/06.mp3"));
fl.push_back(TagLib::FileRef("/tmp/simple/07.mp3"));
fl.push_back(TagLib::FileRef("/tmp/simple/08.mp3"));
fl.push_back(TagLib::FileRef("/tmp/simple/09.mp3"));
fl.push_back(TagLib::FileRef("/tmp/simple/10.mp3"));
fl.push_back(TagLib::FileRef("/tmp/simple/11.mp3"));
fl.push_back(TagLib::FileRef("/tmp/simple/12.mp3"));

    FileDisc fd(fl);
    
    Offsets offsets;
    
    offsets = fd.generateOffsets(150);
    
    std::cout << "Length: " << offsets.length << std::endl;
    
    std::cout << "Length of offset_list: " << offsets.offset_list.size() << std::endl;
    
    for_each(offsets.offset_list.begin(), offsets.offset_list.end(), print<int>("Item "));

    CDDBQuery q(offsets.length, offsets.offset_list);
    
    q.query();
    
    const Discs &d = q.getResults();
    
    for_each(d.begin(), d.end(), print_disc(std::cout));

    fd.tagCDDBDiscByBestFit(d.front());
    
    return 0;
}